@@ -68,6 +68,18 @@ function parseArrival(res, line, year) {
     return true;
 }
 
+function convertToBus(res) {
+    res['@type'] = 'BusReservation';
+    res.reservationFor['@type'] = 'BusTrip';
+    res.reservationFor.departureBusStop = res.reservationFor.departureStation;
+    res.reservationFor.departureBusStop['@type'] = 'BusStop';
+    res.reservationFor.arrivalBusStop = res.reservationFor.arrivalStation;
+    res.reservationFor.arrivalBusStop['@type'] = 'BusStop';
+    res.reservationFor.busName = res.reservationFor.trainName;
+    res.reservationFor.busNumber = res.reservationFor.trainNumber;
+    res.reservedTicket.ticketedSeat = undefined;
+}
+
 function parseLegs(text, year, compact) {
     var reservations = new Array();
     var lines = text.split('\n');
@@ -177,23 +189,71 @@ function parseTicket(text, uic918ticket) {
     // international tickets have the booking reference somewhere on the side, so we don't really know
     // where it is relative to the itinerary
     var bookingRef = text.match(/(?:Auftragsnummer|Auftrag \(NVS\)):\s*([A-Z0-9]{6,9})\n/);
-    for (var i = 0; i < reservations.length; ++i)
+    for (var i = 0; i < reservations.length; ++i) {
         reservations[i].reservationNumber = bookingRef[1];
+
+        if (reservations[i].reservationFor.trainNumber && reservations[i].reservationFor.trainNumber.startsWith("Bus")) {
+            convertToBus(reservations[i]);
+        }
+    }
+    return reservations;
+}
+
+function parseReservation(text) {
+    var reservations = Array();
+    var idx = 0;
+
+    while (true) {
+        var dep = text.substr(idx).match(/  (\S.*\S) +(\d\d.\d\d.\d\d) +ab (\d\d:\d\d)  +(.*?)  +([A-Z].*\S)  +(.*)\n/);
+        if (!dep)
+            return reservations;
+        idx += dep.index + dep[0].length;
+        var arr = text.substr(idx).match(/  (\S.*\S) +an (\d\d:\d\d)  +(.*?)  +(.*)\n/);
+        if (!arr)
+            return reservations;
+
+        var res = JsonLd.newTrainReservation();
+        res.reservationFor.departureStation.name = dep[1];
+        res.reservationFor.departureTime = JsonLd.toDateTime(dep[2] + dep[3], "dd.MM.yyhh:mm", "de");
+        res.reservationFor.trainNumber = dep[5];
+        res.reservationFor.departurePlatform = dep[4];
+        res.reservationFor.arrivalStation.name = arr[1];
+        res.reservationFor.arrivalTime = JsonLd.toDateTime(dep[2] + arr[2], "dd.MM.yyhh:mm", "de");
+        res.reservationFor.arrivalPlatform = arr[3];
+        reservations.push(res);
+
+        seatText = dep[6] + " " + arr[4];
+        var seat = seatText.match(/(\d)\. Klasse, Wg\. (\d+), Pl. (.*?),/);
+        if (seat) {
+            res.reservedTicket.ticketedSeat.seatingType = seat[1];
+            res.reservedTicket.ticketedSeat.seatSection = seat[2];
+            res.reservedTicket.ticketedSeat.seatNumber = seat[3];
+        }
+
+        idx += arr.index + arr[0].length;
+    }
+
     return reservations;
 }
 
 function parsePdf(pdf) {
+    if (Context.pdfPageNumber < 0) { // no barcode
+        return parseReservation(pdf.pages[0].text);
+    }
+
     var page = pdf.pages[Context.pdfPageNumber];
     var uic918ticket = Barcode.decodeUic9183(Context.barcode);
 
     var reservations = parseTicket(page.text, uic918ticket);
     for (var i = 0; i < reservations.length; ++i) {
         reservations[i].reservedTicket.ticketToken = "aztecbin:" + Barcode.toBase64(Context.barcode);
-        reservations[i].reservedTicket.ticketedSeat.seatingType = uic918ticket.seatingType;
         if (Context.data && Context.data.length > 0) {
             reservations[i].reservedTicket.name = Context.data[0].reservedTicket.name;
+            reservations[i].underName = Context.data[0].underName;
+            if (reservations[i].reservedTicket.ticketedSeat) {
+                reservations[i].reservedTicket.ticketedSeat.seatingType = Context.data[0].reservedTicket.ticketedSeat.seatingType;
+            }
         }
-        reservations[i].underName = JsonLd.toJson(uic918ticket.person);
     }
 
     return reservations;
