@@ -25,6 +25,7 @@
 #include <QJsonArray>
 #include <QJsonObject>
 #include <QLocale>
+#include <QMetaProperty>
 #include <QRegularExpression>
 #include <QUrl>
 
@@ -81,8 +82,8 @@ QJSValue JsApi::JsonLd::newFlightReservation() const
 
 QJSValue JsApi::JsonLd::newTrainReservation() const
 {
-    const auto dep = newObject(QStringLiteral("TrainStation"));
-    const auto arr = newObject(QStringLiteral("TrainStation"));
+    const auto dep = newPlace(QStringLiteral("TrainStation"));
+    const auto arr = newPlace(QStringLiteral("TrainStation"));
     const auto person = newObject(QStringLiteral("Person"));
     const auto seat = newObject(QStringLiteral("Seat"));
 
@@ -257,6 +258,22 @@ QJSValue JsApi::JsonLd::toGeoCoordinates(const QString &mapUrl)
     return {};
 }
 
+QJSValue JsApi::JsonLd::readQDateTime(const QVariant &obj, const QString &propName) const
+{
+    const auto mo = QMetaType::metaObjectForType(obj.userType());
+    if (!mo) {
+        return {};
+    }
+    const auto propIdx = mo->indexOfProperty(propName.toUtf8().constData());
+    if (propIdx < 0) {
+        qWarning() << "Unknown property name:" << mo->className() << propName;
+        return {};
+    }
+    const auto prop = mo->property(propIdx);
+    const auto dt = prop.readOnGadget(obj.constData());
+    return toJson(dt);
+}
+
 void JsApi::JsonLd::setContextDate(const QDateTime& dt)
 {
     m_contextDate = dt;
